import { publicClientConfig } from 'sushi/config';
import { SUSHISWAP_V2_SUBGRAPH_URL } from 'sushi/config/subgraph';
import { SushiSwapProtocol } from 'sushi/types';
import { createPublicClient, erc20Abi } from 'viem';
import { FetchError } from '../../../lib/fetch-error';
import { addChainId } from '../../../lib/modifiers/add-chain-id';
import { convertIdToMultichainId } from '../../../lib/modifiers/convert-id-to-multichain-id';
import { copyIdToAddress } from '../../../lib/modifiers/copy-id-to-address';
import { requestPaged } from '../../../lib/request-paged';
import { graphql } from '../graphql';
export const SushiV2LiquidityPositionsQuery = graphql(`
  query LiquidityPositions($first: Int = 1000, $skip: Int = 0, $block: Block_height, $orderBy: LiquidityPosition_orderBy, $orderDirection: OrderDirection, $where: LiquidityPosition_filter) {
    liquidityPositions(first: $first, skip: $skip, block: $block, orderBy: $orderBy, orderDirection: $orderDirection, where: $where) {
      id
      balance: liquidityTokenBalance
      pair {
        id
      }
      user {
        id
      }
    }
  }
`);
export async function getSushiV2LiquidityPositions({ chainId, ...variables }, options) {
    try {
        const url = `https://${SUSHISWAP_V2_SUBGRAPH_URL[chainId]}`;
        const result = await requestPaged({
            chainId,
            url,
            query: SushiV2LiquidityPositionsQuery,
            variables,
            options,
        });
        const transformed = result.liquidityPositions.map((position) => {
            const pool = {
                ...convertIdToMultichainId(copyIdToAddress(addChainId(chainId, position.pair))),
                protocol: SushiSwapProtocol.SUSHISWAP_V2,
            };
            return {
                id: position.id,
                balance: position.balance,
                pool,
                user: position.user.id,
            };
        });
        const client = createPublicClient(publicClientConfig[chainId]);
        const balances = await client.multicall({
            contracts: transformed.map((pos) => ({
                abi: erc20Abi,
                functionName: 'balanceOf',
                address: pos.pool.address,
                args: [pos.user],
            })),
            allowFailure: false,
        });
        transformed.forEach((pos, i) => {
            pos.balance = String(balances[i]);
        });
        return transformed;
    }
    catch {
        throw new FetchError(chainId, 'Failed to fetch liquidity positions');
    }
}
